<?php

namespace Drupal\simplifying\Services;

use Drupal\Core\Url;
use Drupal\Core\Render\Markup;
use Drupal\Core\Menu\MenuTreeParameters;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Extension\ModuleHandler;
use Drupal\Core\Menu\MenuLinkTree;
use Drupal\Core\Routing\CurrentRouteMatch;

/**
 * Class Toolbar.
 *
 * @package Drupal\simplifying
 */
class Toolbar {

  use StringTranslationTrait;

  /**
   * Add services settings actions.
   *
   * @var \Drupal\simplifying\Services\SettingsActions
   */
  protected $settingsactions;

  /**
   * Add module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandler
   */
  protected $modulehandler;

  /**
   * Add menu link tree.
   *
   * @var \Drupal\Core\Menu\MenuLinkTree
   */
  protected $menutree;

  /**
   * Add current route match.
   *
   * @var \Drupal\Core\Routing\CurrentRouteMatch
   */
  protected $currentroutematch;

  /**
   * Add __construct.
   *
   * @param \Drupal\simplifying\Services\SettingsActions $settingsactions
   *
   *   Add SettingsActions.
   * @param \Drupal\Core\Extension\ModuleHandler $modulehandler
   *
   *   Add ModuleHandler.
   * @param \Drupal\Core\Menu\MenuLinkTree $menutree
   *
   *   Add MenuLinkTree.
   * @param \Drupal\Core\Routing\CurrentRouteMatch $currentroutematch
   *
   *   Add CurrentRouteMatch.
   */
  public function __construct(SettingsActions $settingsactions, ModuleHandler $modulehandler, MenuLinkTree $menutree, CurrentRouteMatch $currentroutematch) {
    $this->settingsactions = $settingsactions;
    $this->modulehandler = $modulehandler;
    $this->menutree = $menutree;
    $this->currentroutematch = $currentroutematch;
  }

  /**
   * Get settings toolbar form fields.
   */
  public function formFields(&$form, $form_state) {
    $form['toolbar_design_wrapper'] = [
      '#type'  => 'details',
      '#title' => $this->t('Toolbar design'),
      '#group' => 'tabs',
    ];
    $defs = $this->settingsactions->getSettings('design');

    $form['toolbar_design_wrapper']['small_button'] = [
      '#type'          => 'checkbox',
      '#title'         => $this->t('Small button'),
      '#parents'       => ['design', 'small_button'],
      '#default_value' => !empty($defs['small_button']) ? 1 : 0,
    ];
    $form['toolbar_design_wrapper']['top_background'] = [
      '#type'          => 'color',
      '#title'         => $this->t('Top menu background'),
      '#parents'       => ['design', 'top_background'],
      '#default_value' => !empty($defs['top_background']) ? $defs['top_background'] : '#3A9F00',
    ];
    $form['toolbar_design_wrapper']['top_color'] = [
      '#type'          => 'color',
      '#title'         => $this->t('Top menu color'),
      '#parents'       => ['design', 'top_color'],
      '#default_value' => !empty($defs['top_color']) ? $defs['top_color'] : '#000000',
    ];
    $form['toolbar_design_wrapper']['submenu_background'] = [
      '#type'          => 'color',
      '#title'         => $this->t('Submenu background'),
      '#parents'       => ['design', 'submenu_background'],
      '#default_value' => !empty($defs['submenu_background']) ? $defs['submenu_background'] : '#555555',
    ];

    $form['toolbar_tabs_wrapper'] = [
      '#type'  => 'details',
      '#title' => $this->t('Toolbar tabs'),
      '#group' => 'tabs',
    ];
    $form['toolbar_tabs_wrapper']['toolbar_tabs'] = [
      '#type'          => 'checkboxes',
      '#title'         => $this->t('Toolbar tabs'),
      '#title_display' => 'invisible',
      '#options'       => $this->getDefaultToolbarTabs(),
      '#default_value' => $this->getToolbarTabs(),
    ];

    $menu_tree_parameters = new MenuTreeParameters();
    $tree = $this->menutree->load('admin', $menu_tree_parameters);
    $manipulators = [
      ['callable' => 'menu.default_tree_manipulators:generateIndexAndSort'],
    ];
    $tree = $this->menutree->transform($tree, $manipulators);
    $options = [];
    $this->treeOptions($tree, $options);
    $defs = $this->settingsactions->getSettings('menu_links');

    $form['menu_links'] = [
      '#type'  => 'details',
      '#title' => $this->t('Menu links'),
      '#group' => 'tabs',
    ];

    foreach ($options as $val) {
      $form['menu_links'][$val['path']] = [
        '#prefix'  => Markup::create('<div style="padding-left: ' . (($val['depth'] - 1) * 20) . 'px;">'),
        '#suffix'  => '</div>',
        '#type'    => 'checkbox',
        '#parents' => ['menu_links', $val['path']],
        '#title'   => $val['title'],
      ];
      if (in_array($val['path'], $defs)) {
        $form['menu_links'][$val['path']]['#default_value'] = 1;
      }
    }
  }

  /**
   * Get menu tree options.
   */
  public function treeOptions($tree, &$options) {
    if (isset($tree['system.admin'])) {
      $tree = ['system.admin' => $tree['system.admin']];
    }
    foreach ($tree as $rout => $val) {
      if (!empty($val->link)) {
        $link = $val->link;
        $path = Url::fromRoute($link->getRouteName(), $link->getRouteParameters(), $link->getOptions())->getInternalPath();
        if (!empty($path)) {
          $title = $link->getTitle();
          $options[$rout] = [
            'title' => $title,
            'depth' => $val->depth,
            'path' => $path,
            'rout' => $rout,
          ];
        }
      }
      if (!empty($val->subtree)) {
        $this->treeOptions($val->subtree, $options);
      }
    }
  }

  /**
   * Add changes for toolbar.
   */
  public function toolbarTabs(&$items) {
    $design = $this->settingsactions->getSettings('design');
    $route_name = $this->currentroutematch->getRouteName();

    $class = ['toolbar-icon', 'toolbar-icon-simplifying_switch'];
    $text = $this->t('Full administration');
    if (!empty($_COOKIE['simplifying'])) {
      $class[] = 'is-active';
      $text = $this->t('Small administration');
    }
    $items['simplifying'] = [
      '#type'               => 'toolbar_item',
      '#wrapper_attributes' => [
        'class' => ['toolbar_simplifying', 'toolbar_simplifying_switch'],
      ],
      '#attached'           => [
        'library' => ['simplifying/simplifying'],
      ],
      '#weight'             => -50,
      'tab'                 => [
        '#type'       => 'html_tag',
        '#tag'        => 'button',
        '#value'      => $text,
        '#attributes' => [
          'class'   => $class,
          'onclick' => 'SimplifyingSwitch(this)',
        ],
      ],
    ];
    if (!empty($design['small_button'])) {
      $items['simplifying']['tab']['#attributes']['class'][] = 'small_button';
      $items['simplifying']['tab']['#attributes']['title'] = $text;
    }

    $items['simplifying']['#attached']['drupalSettings']['simplifying_menu_links'] = $this->settingsactions->getSettings('menu_links');

    $toolbar_tabs = $this->getToolbarTabs();
    // Alter to hide the toolbar tabs.
    $this->modulehandler->alter('simplifying_hide_toolbar_tabs', $toolbar_tabs);

    $items['simplifying']['#attached']['drupalSettings']['simplifying_menu_tabs'] = $toolbar_tabs;

    $styles = [];
    if (!empty($design['top_background'])) {
      $styles[] = '#toolbar-bar{ background-color: ' . $design['top_background'] . '; }';
    }
    if (!empty($design['top_color'])) {
      $styles[] = '.toolbar .toolbar-bar .toolbar-item{ color: ' . $design['top_color'] . '; }';
    }
    if (!empty($design['submenu_background'])) {
      $styles[] = '#toolbar-item-administration-tray{ background-color: ' . $design['submenu_background'] . '; }';
    }
    if (!empty($styles)) {
      $items['simplifying']['tab']['#suffix'] = Markup::create('<style>' . implode($styles) . '</style>');
    }

    $class = ['toolbar-icon', 'toolbar-icon-simplifying_services'];
    if ($route_name == 'simplifying.services') {
      $class[] = 'is-active';
    }
    $text = $this->t('Order additional services');
    $items['services'] = [
      '#type'               => 'toolbar_item',
      '#wrapper_attributes' => [
        'class' => ['toolbar_simplifying', 'toolbar_simplifying_services'],
      ],
      '#weight'             => -49,
      'tab'                 => [
        '#type'       => 'link',
        '#title'      => $text,
        '#url'        => Url::fromRoute('simplifying.services'),
        '#attributes' => ['class' => $class],
      ],
    ];
    if (!empty($design['small_button'])) {
      $items['services']['tab']['#attributes']['class'][] = 'small_button';
      $items['services']['#wrapper_attributes']['title'] = $text;
    }

    $class = ['toolbar-icon', 'toolbar-icon-simplifying_training'];
    if ($route_name == 'simplifying.training') {
      $class[] = 'is-active';
    }
    $text = $this->t('Training');
    $items['training'] = [
      '#type'               => 'toolbar_item',
      '#wrapper_attributes' => [
        'class' => ['toolbar_simplifying', 'toolbar_simplifying_training'],
      ],
      '#weight'             => -48,
      'tab'                 => [
        '#type'       => 'link',
        '#title'      => $text,
        '#url'        => Url::fromRoute('simplifying.training'),
        '#attributes' => ['class' => $class],
      ],
    ];
    if (!empty($design['small_button'])) {
      $items['training']['tab']['#attributes']['class'][] = 'small_button';
      $items['training']['#wrapper_attributes']['title'] = $text;
    }
  }

  /**
   * Get settings toolbar tabs.
   */
  public function getToolbarTabs() {
    $toolbar_tabs = $this->settingsactions->getSettings('toolbar_tabs');
    if (empty($toolbar_tabs)) {
      $toolbar_tabs = ['devel'];
    }
    return array_filter($toolbar_tabs);
  }

  /**
   * Get default toolbart tabs.
   */
  public function getDefaultToolbarTabs() {
    return [
      'home'           => $this->t('Back to site'),
      'administration' => $this->t('Manage'),
      'shortcuts'      => $this->t('Shortcuts'),
      'user'           => $this->t('User'),
      'devel'          => $this->t('Devel'),
      'contextual'     => $this->t('Edit'),
    ];
  }

}
